"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const create_provider_1 = require("./create-provider");
const devtools_1 = require("./devtools");
function createStore(initialState, actions, extraArgument) {
    let getState;
    let setState;
    const Context = react_1.default.createContext(initialState);
    const Provider = create_provider_1.createProvider((providerGetState, providerSetState) => {
        getState = providerGetState;
        setState = (state) => {
            if (typeof state === 'function') {
                return state(providerGetState, providerSetState, extraArgument);
            }
            else {
                devToolsLogger && devToolsLogger('state update', state);
                providerSetState(state);
            }
        };
    }, Context.Provider, initialState);
    const devToolsLogger = typeof window !== 'undefined' &&
        window.__REDUX_DEVTOOLS_EXTENSION__
        ? devtools_1.createDevToolsLogger()
        : undefined;
    const wrappedActions = {};
    Object.keys(actions)
        .forEach(key => {
        const originalFn = actions[key];
        const wrappedFn = (...args) => {
            const result = originalFn(...args);
            if (!getState || !setState) {
                // tslint:disable-next-line:no-console
                console.error('<Provider /> is not initialized');
                return;
            }
            if (typeof result === 'function') {
                devToolsLogger && devToolsLogger(`thunk — ${key}`, getState());
                return result(getState, setState, extraArgument);
            }
            else {
                // Never-ever goes here
                devToolsLogger && devToolsLogger(key, result);
                setState(result);
                return;
            }
        };
        wrappedActions[key] = wrappedFn;
    });
    return {
        Context,
        Provider,
        Consumer: Context.Consumer,
        actions: wrappedActions,
        getState: () => getState ? getState() : initialState,
    };
}
exports.createStore = createStore;
